/**
 * JASMINe
 * Copyright (C) 2008-2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MBeanCmdManager.java 5427 2009-09-25 07:24:02Z jlegrand $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.mbeancmdmanager.service;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.ow2.jasmine.monitoring.eos.mbeancmdmanager.service.exceptions.MBeanCmdManagerException;



/**
 * Manage MBeanCmd instances.
 *
 * @author Guillaume Renault
 *
 */
public class MBeanCmdManager implements Serializable {

    /**
     *
     */
    private static final long serialVersionUID = 48822901471540481L;

    /**
     * Default constructor
     */
    public MBeanCmdManager() {

    }

    /**
     * MBeanServerConnection instance.
     */
    private transient MBeanServerConnection mbscnx = null;

    /**
     * Performs a connection to the JMX server.
     *
     * @param jmxUrl the JMX url of the server.
     * @return true if the connection occured, false otherwise.
     */
    private boolean connect(final String jmxUrl) {
        System.out.println("Connecting to : " + jmxUrl + "...");
        try {
            JMXServiceURL url = new JMXServiceURL(jmxUrl);
            JMXConnector jmxc = JMXConnectorFactory.connect(url, null);
            mbscnx = jmxc.getMBeanServerConnection();
            return true;

        } catch (Exception e) {
            return false;
        }
    }

    /**
     * Add a new MBeanCmd command.
     *
     * @param command the command to launch.
     * @param jmxUrl the JMX url server to connect to.
     * @return a unique identifier for the MBeanCmd command.
     * @throws MBeanCmdManagerException ...
     */
    public String newCommand(final String command, final String jmxUrl) throws MBeanCmdManagerException {
        if (!this.connect(jmxUrl)) {
            throw new MBeanCmdManagerException("Cannot connect to the MBeanCmd manager service located on " + jmxUrl);
        }
        ObjectName manager = null;
        String result = command;
        try {
            Set<ObjectName> objNames = mbscnx.queryNames(null, null);
            for (ObjectName obj : objNames) {
                if (obj.getCanonicalName().contains("type=EventSwitch") && obj.getCanonicalName().contains("name=manager")) {
                    manager = obj;
                    break;
                }
            }

        } catch (IOException e) {
            throw new MBeanCmdManagerException("No MBean found",e);
        }

        try {
            result = (String) mbscnx.invoke(manager, "startMBeanCmdCommand", new Object[] {command}, new String[] {String.class
                    .getName()});
        } catch (Exception e) {
            throw new MBeanCmdManagerException("Invoke failed ... ",e);
        }
        return result;
    }

    /**
     * Gets all MBeanCmd instances on a specific EventSwitch
     *
     * @param jmxUrl the JMX url server to connect to.
     * @return a list of String representing all launched MBeanCmd instances
     * @throws MBeanCmdManagerException ...
     */
    public List<String> getMBeanCmdInstanceNames(final String jmxUrl) throws MBeanCmdManagerException {
        if (!this.connect(jmxUrl)) {
            throw new MBeanCmdManagerException("Cannot connect to the MBeanCmd manager service located on " + jmxUrl);
        }
        ObjectName manager = null;
        List<String> result = new ArrayList<String>();
        try {
            Set<ObjectName> objNames = mbscnx.queryNames(null, null);
            for (ObjectName obj : objNames) {
                if (obj.getCanonicalName().contains("type=EventSwitch") && obj.getCanonicalName().contains("name=manager")) {
                    manager = obj;
                    break;
                }
            }

        } catch (IOException e) {
            throw new MBeanCmdManagerException("No MBean found");
        }

        String[] r = null;

        try {
            r = (String[]) mbscnx.invoke(manager, "MBeanCmdCommandNames", null, null);
        } catch (Exception e) {
            throw new MBeanCmdManagerException("Invoke failed ... ");
        }

        for (int i = 0; i < r.length; i++) {
            result.add(r[i]);
        }

        return result;
    }

}
